/********************************************************************************/
/*                                                                              */
/*              chat.H                                                          */
/*                                                                              */
/*      External definitions for the CHAT server                                */
/*                                                                              */
/********************************************************************************/

#ifndef CHAT_ALREADY_DEFINED
#define CHAT_ALREADY_DEFINED



/********************************************************************************/
/*                                                                              */
/*      Parameters                                                              */
/*                                                                              */
/********************************************************************************/


const char * const      CHAT_SERVER_NAME = "/u/spr/oodbook/ipc/chatserver";
const char * const      CHAT_SERVER_FILE = "/u/spr/oodbook/ipc/chat.socket";



/********************************************************************************/
/*                                                                              */
/*      Type Definitions                                                        */
/*                                                                              */
/********************************************************************************/


typedef class ChatFactoryInfo *         ChatFactory;
typedef class ChatSessionInfo *         ChatSession;
typedef class ChatCallbackInfo *        ChatCallback;



/********************************************************************************/
/*                                                                              */
/*      ChatSessionInfo -- hold a chat session using a remote server            */
/*                                                                              */
/********************************************************************************/


class ChatSessionInfo {

public:
   virtual ~ChatSessionInfo()                                           { }

   virtual void setCallback(ChatCallback)                               { }

   virtual void setUserName(const char * id) = 0;
   virtual void message(const char * message) = 0;
};



/********************************************************************************/
/*                                                                              */
/*      ChatCallbackInfo -- handle callbacks from a chat session                */
/*                                                                              */
/********************************************************************************/


class ChatCallbackInfo {

public:
   virtual ~ChatCallbackInfo()                                                  { }

   virtual void chatMessage(const char * user,const char * message) = 0;
   virtual void chatUserStart(const char * user)                                { }
   virtual void chatUserEnd(const char * user)                                  { }
   virtual void chatSessionEnd()                                                { }
};




#endif


/* end of chat.H */
