/********************************************************************************/
/*                                                                              */
/*              chat_server.H                                                   */
/*                                                                              */
/*      Local definitions for the CHAT server                                   */
/*                                                                              */
/********************************************************************************/


#ifndef CHAT_SERVER_ALREADY_DEFINED
#define CHAT_SERVER_ALREADY_DEFINED

#include "chat.H"
#include "chat_message.H"

#include "stddef.H"
#include "connect.H"
#include <baum.H>



/********************************************************************************/
/*                                                                              */
/*      Type Definitions                                                        */
/*                                                                              */
/********************************************************************************/


typedef class ChatServerInfo *          ChatServer;
typedef class ChatConnectInfo *         ChatConnect;
typedef class ChatUserCallbackInfo *    ChatUserCallback;
typedef class ChatUserInfo *            ChatUser;
typedef class ChatUserConnectInfo *     ChatUserConnect;

typedef list<ChatUser>                  ChatUserList;
typedef ChatUserList::iterator          ChatUserListIter;



/********************************************************************************/
/*                                                                              */
/*      ChatServerInfo -- main server for chattering                            */
/*                                                                              */
/********************************************************************************/


class ChatServerInfo {

private:
   BAUM x11_context;
   ChatConnect server_connect;
   ChatUserList active_list;

public:
   ChatServerInfo(int,const char**);
   ~ChatServerInfo();

   void process();

public:
   void addUser(ChatUser);
   void removeUser(ChatUser);
   void getMessage(ChatUser,ConstText msg);

   Connect masterConnect() const;
};




/********************************************************************************/
/*                                                                              */
/*      ChatConnectInfo -- connection to the server                             */
/*                                                                              */
/********************************************************************************/


class ChatConnectInfo : public ConnectInfo {

private:
   ChatServer for_server;

public:
   ChatConnectInfo(ChatServer);
   ~ChatConnectInfo();

private:
   void acceptCallback();
};



/********************************************************************************/
/*                                                                              */
/*      ChatUserInfo -- information about a suer                                */
/*                                                                              */
/********************************************************************************/


class ChatUserInfo : public ChatSessionInfo {

private:
   ChatServer for_server;
   ChatUserCallback user_callback;
   ChatUserConnect user_connect;
   StdString user_name;

public:
   ChatUserInfo(ChatServer);
   ~ChatUserInfo();

   void setUserName(const char * id);
   ConstText name() const                       { return user_name.c_str(); }
   ChatUserCallback callback() const            { return user_callback; }

   void message(const char * message);
};




/********************************************************************************/
/*                                                                              */
/*      ChatUserConnectInfo -- server connection to a user                      */
/*                                                                              */
/********************************************************************************/


class ChatUserConnectInfo : public ConnectInfo {

private:
   ChatUser for_user;

public:
   ChatUserConnectInfo(ChatUser,Connect);
   ~ChatUserConnectInfo();

private:
   void closeCallback();
   void messageCallback(int len,void * msg);
};



/********************************************************************************/
/*                                                                              */
/*      ChatUserCallbackInfo -- callback to user application                    */
/*                                                                              */
/********************************************************************************/


class ChatUserCallbackInfo : public ChatCallbackInfo {

private:
   ChatUserConnect user_connect;

public:
   ChatUserCallbackInfo(ChatUserConnect);
   ~ChatUserCallbackInfo();

   void chatMessage(const char * user,const char * message);
   void chatUserStart(const char * user);
   void chatUserEnd(const char * user);
};



#endif


/* end of chat_server.H */














