/********************************************************************************/
/*                                                                              */
/*              chatserver.cxx                                                  */
/*                                                                              */
/*      Server implementation for chat program                                  */
/*                                                                              */
/********************************************************************************/


#include "chat_server.H"



/********************************************************************************/
/*                                                                              */
/*      main -- main server program                                             */
/*                                                                              */
/********************************************************************************/


main(int argc,const char ** argv)
{
   ChatServer cs = new ChatServerInfo(argc,argv);

   cs->process();

   delete cs;

   return 0;
}




/********************************************************************************/
/*                                                                              */
/*      ChatServerInfo constructors/destructores                                */
/*                                                                              */
/********************************************************************************/


ChatServerInfo::ChatServerInfo(int argc,const char ** argv)
{
   x11_context.defaultContext();
   ConnectInfo::setX11Context(x11_context);
   server_connect = NULL;
}



ChatServerInfo::~ChatServerInfo()
{
   delete server_connect;
}



/********************************************************************************/
/*                                                                              */
/*      ChatServerInfo processing methods                                       */
/*                                                                              */
/********************************************************************************/


void
ChatServerInfo::process()
{
   server_connect = new ChatConnectInfo(this);
   server_connect->openServer(CHAT_SERVER_FILE);

   x11_context.mainLoop();
}




/********************************************************************************/
/*                                                                              */
/*      ChatServerInfo user management methods                                  */
/*                                                                              */
/********************************************************************************/


void
ChatServerInfo::addUser(ChatUser cu)
{
   ChatUserListIter uli;

   for (uli = active_list.begin(); uli != active_list.end(); ++uli) {
      (*uli)->callback()->chatUserStart(cu->name());
    }

   active_list.push_back(cu);
}


void
ChatServerInfo::removeUser(ChatUser cu)
{
   ChatUserListIter uli;

   active_list.remove(cu);
   if (active_list.empty()) exit(0);

   for (uli = active_list.begin(); uli != active_list.end(); ++uli) {
      (*uli)->callback()->chatUserEnd(cu->name());
    }
}


void
ChatServerInfo::getMessage(ChatUser cu,ConstText msg)
{
   ChatUserListIter uli;

   for (uli = active_list.begin(); uli != active_list.end(); ++uli) {
      (*uli)->callback()->chatMessage(cu->name(),msg);
    }
}



/********************************************************************************/
/*                                                                              */
/*      ChatServerInfo access methods                                           */
/*                                                                              */
/********************************************************************************/


Connect
ChatServerInfo::masterConnect() const
{
   return server_connect;
}



/********************************************************************************/
/*                                                                              */
/*      ChatConnectInfo constructors/destructors                                */
/*                                                                              */
/********************************************************************************/


ChatConnectInfo::ChatConnectInfo(ChatServer cs)
{
   for_server = cs;
}



ChatConnectInfo::~ChatConnectInfo()
{ }



/********************************************************************************/
/*                                                                              */
/*      ChatConnectInfo callback methods                                        */
/*                                                                              */
/********************************************************************************/


void
ChatConnectInfo::acceptCallback()
{
   ChatUser cu = new ChatUserInfo(for_server);
}



/* end of chatserver.cxx */
