/********************************************************************************/
/*                                                                              */
/*              chatter_local.H                                                 */
/*                                                                              */
/*      Locatl definitions for the chatter client                               */
/*                                                                              */
/********************************************************************************/


#ifndef CHATTER_LOCAL_ALREADY_DEFINED
#define CHATTER_LOCAL_ALREADY_DEFINED


#include "stddef.H"
#include "connect.H"
#include "chat.H"
#include "chat_message.H"

#include <baum_widget.H>




/********************************************************************************/
/*                                                                              */
/*      Forward Type Definitions                                                */
/*                                                                              */
/********************************************************************************/


typedef class ChatterMainInfo *         ChatterMain;
typedef class ChatterCallbackInfo *     ChatterCallback;
typedef class ChatterClientInfo *       ChatterClient;
typedef class ChatterConnectInfo *      ChatterConnect;



/********************************************************************************/
/*                                                                              */
/*      ChatterMainInfo -- main class for chatter program                       */
/*                                                                              */
/********************************************************************************/


class ChatterMainInfo : public BaumCallbackInfo {

private:
   BaumDisplay for_display;
   BaumApplicationShellWidget app_shell;
   BaumScrolledTextWidget text_area;
   BaumTextFieldWidget input_area;
   BaumPushButtonWidget quit_btn;
   ChatterClient chat_client;
   ChatterCallback chat_callback;

public:
   ChatterMainInfo(int argc,const char ** argv);
   ~ChatterMainInfo();

   void process();
   void addOutput(ConstText);

private:
   static void handleQuit(Widget,void *,void *);
   static void handleInput(Widget,void *,void *);

   void getInput();

private:
   void setupWindows();
};



/********************************************************************************/
/*                                                                              */
/*      ChatterCallbackInfo -- callback handler for chat client                 */
/*                                                                              */
/********************************************************************************/


class ChatterCallbackInfo : public ChatCallbackInfo {

private:
   ChatterMain for_main;

public:
   ChatterCallbackInfo(ChatterMain);
   ~ChatterCallbackInfo();

   void chatMessage(const char *,const char *);
   void chatUserStart(const char *);
   void chatUserEnd(const char *);
   void chatSessionEnd();
};



/********************************************************************************/
/*                                                                              */
/*      ChatterClientInfo -- client end of the chat connection                  */
/*                                                                              */
/********************************************************************************/


class ChatterClientInfo : public ChatSessionInfo {

private:
   ChatterConnect comm_connect;
   ChatCallback user_callback;

public:
   ChatterClientInfo();
   ~ChatterClientInfo();

   void setCallback(ChatCallback cb);
   void setUserName(const char * id);
   void message(const char * msg);

public:
   ChatCallback callback() const                { return user_callback; }
};



/********************************************************************************/
/*                                                                              */
/*      ChatterConnectInfo -- connection from the client end                    */
/*                                                                              */
/********************************************************************************/


class ChatterConnectInfo : public ConnectInfo {

private:
   ChatCallback client_callback;

public:
   ChatterConnectInfo();
   ~ChatterConnectInfo();

   void setCallback(ChatCallback);

   void closeCallback();
   void messageCallback(int,void *);
};




#endif


/* end of chatter_local.H */
