/********************************************************************************/
/*                                                                              */
/*              chatterclient.cxx                                               */
/*                                                                              */
/*      Implementation of the chat client for CHATTER                           */
/*                                                                              */
/********************************************************************************/


#include "chatter_local.H"



/********************************************************************************/
/*                                                                              */
/*      ChatterClientInfo constructors/destructors                              */
/*                                                                              */
/********************************************************************************/


ChatterClientInfo::ChatterClientInfo()
{
   Integer i;

   user_callback = NULL;
   comm_connect = new ChatterConnectInfo();

   if (comm_connect->openClient(CHAT_SERVER_FILE)) return;

   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << CHAT_SERVER_NAME << " &" << ends;
   system(buf);

   for (i = 0; i < 10; ++i) {
      if (comm_connect->openClient(CHAT_SERVER_FILE)) break;
      sleep(1);
    }
   if (i >= 10) {
      cerr << "CHAT: can't connect to server" << endl;
      exit(1);
    }
}



ChatterClientInfo::~ChatterClientInfo()
{
   comm_connect->close();

   delete comm_connect;
   delete user_callback;
}



/********************************************************************************/
/*                                                                              */
/*      ChatterClientInfo client interface                                      */
/*                                                                              */
/********************************************************************************/


void
ChatterClientInfo::setCallback(ChatCallback cb)
{
   user_callback = cb;

   comm_connect->setCallback(cb);
}



void
ChatterClientInfo::setUserName(const char * id)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << CHAT_MSG_SET_USER << " " << id << ends;

   comm_connect->sendMessage(buf);
}



void
ChatterClientInfo::message(const char * msg)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << CHAT_MSG_CLIENT << " " << msg << ends;

   comm_connect->sendMessage(buf);
}




/********************************************************************************/
/*                                                                              */
/*      ChatterConnectInfo constructors/destructors                             */
/*                                                                              */
/********************************************************************************/


ChatterConnectInfo::ChatterConnectInfo()
{
   client_callback = NULL;
}



ChatterConnectInfo::~ChatterConnectInfo()                             { }




/********************************************************************************/
/*                                                                              */
/*      ChatterConnectInfo callback handlers                                    */
/*                                                                              */
/********************************************************************************/


void
ChatterConnectInfo::setCallback(ChatCallback cb)
{
   client_callback = cb;
}



void
ChatterConnectInfo::closeCallback()
{
   if (client_callback != NULL) client_callback->chatSessionEnd();
}



void
ChatterConnectInfo::messageCallback(int len,void * msg)
{
   Text mtxt = (Text) msg;
   ConstText args[CHAT_MSG_MAX_ARGS];
   Integer act = 0;

   if (client_callback == NULL) return;

   Text mp = &mtxt[2];
   while (act < CHAT_MSG_MAX_ARGS) {
      args[act++] = mp;
      while (*mp != 0 && *mp != CHAT_MSG_SEPARATOR) ++mp;
      if (*mp == 0) break;
      *mp++ = 0;
    }

   switch (mtxt[0]) {
      case CHAT_MSG_SERVER :
         client_callback->chatMessage(args[0],args[1]);
         break;
      case CHAT_MSG_NEW_USER :
         client_callback->chatUserStart(args[0]);
         break;
      case CHAT_MSG_REMOVE_USER :
         client_callback->chatUserEnd(args[0]);
         break;
      default :
         break;
    }
}



/* end of chatterclient.cxx */
