/********************************************************************************/
/*                                                                              */
/*              chattermain.cxx                                                 */
/*                                                                              */
/*      Main program and logic for chatter program                              */
/*                                                                              */
/********************************************************************************/


#include "chatter_local.H"




/********************************************************************************/
/*                                                                              */
/*      main -- main program                                                    */
/*                                                                              */
/********************************************************************************/


main(int argc,const char ** argv)
{
   ChatterMain cm = new ChatterMainInfo(argc,argv);

   cm->process();

   delete cm;

   return 0;
}



/********************************************************************************/
/*                                                                              */
/*      ChatterMainInfo constructors/destructors                                */
/*                                                                              */
/********************************************************************************/


ChatterMainInfo::ChatterMainInfo(int argc,const char ** argv)
{
   for_display.open(argc,argv);
   ConnectInfo::setX11Context(for_display);

   chat_client = NULL;
   chat_callback = NULL;
}



ChatterMainInfo::~ChatterMainInfo()
{
   app_shell.destroy();
   delete chat_client;
   delete chat_callback;
}



/********************************************************************************/
/*                                                                              */
/*      ChatterMainInfo processing methods                                      */
/*                                                                              */
/********************************************************************************/


void
ChatterMainInfo::process()
{
   chat_callback = new ChatterCallbackInfo(this);
   chat_client = new ChatterClientInfo;
   chat_client->setCallback(chat_callback);
   chat_client->setUserName(getenv("USER"));

   setupWindows();

   for_display.mainLoop();
}



/********************************************************************************/
/*                                                                              */
/*      ChatterMainInfo window setup methods                                    */
/*                                                                              */
/********************************************************************************/


void
ChatterMainInfo::setupWindows()
{
   BaumFormWidget form;
   BaumFrameWidget frame;
   BaumSeparatorWidget sep1,sep2;
   BaumCoreWidget par;

   app_shell.createRoot(for_display);

   frame.createManaged("chatterFrame",app_shell);
   form.createManaged("chatterForm",frame);
   form.width(400);
   form.height(400);

   quit_btn.createManaged("chatterQuit",form);
   quit_btn.labelString("Quit");
   quit_btn.bottomAttachment(Baum::ATTACH_FORM);
   quit_btn.activateCallback(handleQuit,this);

   sep2.createManaged("chatterSep1",form);
   sep2.orientation(Baum::HORIZONTAL);
   sep2.leftAttachment(Baum::ATTACH_FORM);
   sep2.rightAttachment(Baum::ATTACH_FORM);
   sep2.bottomAttachment(Baum::ATTACH_WIDGET,quit_btn);

   input_area.createManaged("chatterInput",form);
   input_area.leftAttachment(Baum::ATTACH_FORM);
   input_area.rightAttachment(Baum::ATTACH_FORM);
   input_area.bottomAttachment(Baum::ATTACH_WIDGET,sep2);
   input_area.activateCallback(handleInput,this);

   sep1.createManaged("chatterSep1",form);
   sep1.orientation(Baum::HORIZONTAL);
   sep1.leftAttachment(Baum::ATTACH_FORM);
   sep1.rightAttachment(Baum::ATTACH_FORM);
   sep1.bottomAttachment(Baum::ATTACH_WIDGET,input_area);

   text_area.createManaged("chatterText",form);
   text_area.editable(FALSE);

   par = text_area.parent();
   par.leftAttachment(Baum::ATTACH_FORM);
   par.rightAttachment(Baum::ATTACH_FORM);
   par.topAttachment(Baum::ATTACH_FORM);
   par.bottomAttachment(Baum::ATTACH_WIDGET,sep1);

   app_shell.realize();
}



/********************************************************************************/
/*                                                                              */
/*      ChatterMainInfo event callback methods                                  */
/*                                                                              */
/********************************************************************************/


void
ChatterMainInfo::handleQuit(Widget,void * ud,void *)
{
   ChatterMain cm = (ChatterMain) ud;

   delete cm;

   exit(0);
}


void
ChatterMainInfo::handleInput(Widget,void * ud,void *)
{
   ChatterMain cm = (ChatterMain) ud;

   cm->getInput();
}



void
ChatterMainInfo::getInput()
{
   ConstText s = input_area.string();

   chat_client->message(s);

   input_area.string("");
}



/********************************************************************************/
/*                                                                              */
/*      ChatterMainInfo display methods                                         */
/*                                                                              */
/********************************************************************************/


void
ChatterMainInfo::addOutput(ConstText msg)
{
   XmTextPosition p = text_area.lastPosition();
   text_area.insert(p,msg);
}




/********************************************************************************/
/*                                                                              */
/*      ChatterCallbackInfo constructors/destructors                            */
/*                                                                              */
/********************************************************************************/


ChatterCallbackInfo::ChatterCallbackInfo(ChatterMain cm)
{
   for_main = cm;
}


ChatterCallbackInfo::~ChatterCallbackInfo()                             { }




/********************************************************************************/
/*                                                                              */
/*      ChatterCallbackInfo callback handlers                                   */
/*                                                                              */
/********************************************************************************/


void
ChatterCallbackInfo::chatMessage(const char * user,const char * msg)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << user << ": " << msg << "\n" << ends;
   for_main->addOutput(buf);
}




void
ChatterCallbackInfo::chatUserStart(const char * user)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << "***" << ": " << "User " << user << " has entered the chat\n" << ends;
   for_main->addOutput(buf);
}




void
ChatterCallbackInfo::chatUserEnd(const char * user)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << "***" << ": " << "User " << user << " has left the chat\n" << ends;
   for_main->addOutput(buf);
}



void
ChatterCallbackInfo::chatSessionEnd()
{
   for_main->addOutput("Chat session has ended\n");
}



/* end of chattermain.cxx */
