/********************************************************************************/
/*                                                                              */
/*              chatuser.cxxxx                                                  */
/*                                                                              */
/*      Implementation of a specific user for the chat server                   */
/*                                                                              */
/********************************************************************************/


#include "chat_server.H"



/********************************************************************************/
/*                                                                              */
/*      ChatUserInfo constructor/destructors                                    */
/*                                                                              */
/********************************************************************************/


ChatUserInfo::ChatUserInfo(ChatServer cs)
{
   for_server = cs;
   user_connect = new ChatUserConnectInfo(this,for_server->masterConnect());
   user_name = "???";
   user_callback = new ChatUserCallbackInfo(user_connect);
}



ChatUserInfo::~ChatUserInfo()
{
   for_server->removeUser(this);
   delete user_connect;
   delete user_callback;
}



/********************************************************************************/
/*                                                                              */
/*      ChatUserInfo action methods                                             */
/*                                                                              */
/********************************************************************************/


void
ChatUserInfo::setUserName(const char * id)
{
   user_name = id;
   for_server->addUser(this);
}



void
ChatUserInfo::message(const char * msg)
{
   for_server->getMessage(this,msg);
}



/********************************************************************************/
/*                                                                              */
/*      ChatUserConnectInfo constructors/destructors                            */
/*                                                                              */
/********************************************************************************/


ChatUserConnectInfo::ChatUserConnectInfo(ChatUser cu,Connect master)
{
   for_user = cu;
   master->openAccept(this);
}


ChatUserConnectInfo::~ChatUserConnectInfo()                             { }




/********************************************************************************/
/*                                                                              */
/*      ChatUserConnectInfo callback methods                                    */
/*                                                                              */
/********************************************************************************/


void
ChatUserConnectInfo::closeCallback()
{
   delete for_user;
}



void
ChatUserConnectInfo::messageCallback(int len,void * msg)
{
   Text tmsg = (Text) msg;
   ChatMsgType mtype = tmsg[0];
   Text mbody = &tmsg[2];

   switch (mtype) {
      case CHAT_MSG_SET_USER :
         for_user->setUserName(mbody);
         break;
      case CHAT_MSG_CLIENT :
         for_user->message(mbody);
         break;
      default :
         break;
    }
}



/********************************************************************************/
/*                                                                              */
/*      ChatUserCallbackInfo constructors/destructors                           */
/*                                                                              */
/********************************************************************************/


ChatUserCallbackInfo::ChatUserCallbackInfo(ChatUserConnect conn)
{
   user_connect = conn;
}


ChatUserCallbackInfo::~ChatUserCallbackInfo()               { }




/********************************************************************************/
/*                                                                              */
/*      ChatUserCallbackInfo callback handlers                                  */
/*                                                                              */
/********************************************************************************/


void
ChatUserCallbackInfo::chatMessage(const char * u,const char * m)
{
   Character buf[10240];
   ostrstream ost(buf,10240);

   ost << CHAT_MSG_SERVER << " " << u << CHAT_MSG_SEPARATOR << m << ends;

   user_connect->sendMessage(buf);
}



void
ChatUserCallbackInfo::chatUserStart(const char * u)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << CHAT_MSG_NEW_USER << " " << u << ends;

   user_connect->sendMessage(buf);
}



void
ChatUserCallbackInfo::chatUserEnd(const char * u)
{
   Character buf[1024];
   ostrstream ost(buf,1024);

   ost << CHAT_MSG_REMOVE_USER << " " << u << ends;

   user_connect->sendMessage(buf);
}




/* end of chatuser.cxx */
