/********************************************************************************/
/*                                                                              */
/*              simple_socket.H                                                 */
/*                                                                              */
/*      General wrapper for socket calls                                        */
/*                                                                              */
/********************************************************************************/


#ifndef SIMPLE_SOCKET_ALREADY_INCLUDED
#define SIMPLE_SOCKET_ALREADY_INCLUDED


#include <bool.h>
#include <bstring.h>
#include <netinet/in.h>
#include <X11/Intrinsic.h>



/********************************************************************************/
/*                                                                              */
/*      Type Definitions                                                        */
/*                                                                              */
/********************************************************************************/


typedef class SimpleSocketInfo *        SimpleSocket;



/********************************************************************************/
/*                                                                              */
/*      SimpleSocket -- general socket wrapper                                  */
/*                                                                              */
/********************************************************************************/


class SimpleSocketInfo {

public:
   typedef int Error;

private:
   int socket_id;

   static Error last_error;

public:
   SimpleSocketInfo();
   virtual ~SimpleSocketInfo();

   bool create();                               // create a socket
   void close();                                // close the socket

   int send(const void * buf,int bln);          // send a message
   int receive(void * buf,int bln);             // receive a message

   bool bind(int port = 0);                     // bind socket to local port
   bool getSockName(char * host,int& port);
   bool enableNonblocking(bool);                // enable/disable nonblocking socket
   bool listen(int n = 5);                      // allow socket to listen
   bool accept(SimpleSocket);
   bool connect(char * host,int port);          // client connect to server

   Error getLastError()                         { return last_error; }

public:
   virtual void onAccept();                     // accept available on the socket
   virtual void onClose();                      // other end closed the socket
   virtual void onConnect();                    // connection ready on this socket
   virtual void onReceive();                    // socket has data to read
   virtual void onSend();                       // socket ready to send

public:
   void waitForEvent();

   static XtAppContext useX11(XtAppContext = NULL);
};




#endif

/* end of socket.H */
