/********************************************************************************/
/*                                                                              */
/*              simple_thread.H                                                 */
/*                                                                              */
/*      Include file for simple use of threads                                  */
/*                                                                              */
/********************************************************************************/


#ifndef SIMPLE_THREAD_ALREADY_INCLUDED
#define SIMPLE_THREAD_ALREADY_INCLUDED

#ifndef _REENTRANT
#define _REENTRANT
#endif

#include <thread.h>
#include <signal.h>
#include <bool.h>



/********************************************************************************/
/*                                                                              */
/*      SimpleThread -- basic thread wrapper                                    */
/*                                                                              */
/********************************************************************************/


class SimpleThread {

private:
   thread_t thread_id;

public:
   SimpleThread();
   ~SimpleThread();

   bool createThread(bool suspended = 0,int stacksize = 0);

   int getThreadPriority();
   bool setThreadPriority(int);

   int suspendThread();
   int resumeThread();

public:
   virtual int run();
};



/********************************************************************************/
/*                                                                              */
/*      SimpleMutex -- basic mutex                                              */
/*                                                                              */
/********************************************************************************/


class SimpleMutex {

private:
   mutex_t mutex_id;

public:
   SimpleMutex(int on = 0);
   ~SimpleMutex();

   bool lock();
   bool unlock();
};


/********************************************************************************/
/*                                                                              */
/*      SimpleSemaphore -- basic semaphore                                      */
/*                                                                              */
/********************************************************************************/


class SimpleSemaphore {

private:
   sema_t sema_id;

public:
   SimpleSemaphore(unsigned int count = 0);
   ~SimpleSemaphore();

   bool lock();
   bool unlock();
};



#endif

/* end of simple_thread.H */
