/********************************************************************************/
/*                                                                              */
/*              simplethread.cxx                                                */
/*                                                                              */
/*      Implementation of the simple thread and mutex classes                   */
/*                                                                              */
/********************************************************************************/


#include "simple_thread.H"
#include "stddef.H"



/********************************************************************************/
/*                                                                              */
/*      SimpleThread constructors/destructors                                   */
/*                                                                              */
/********************************************************************************/


SimpleThread::SimpleThread()
{
   thread_id = 0;
}



SimpleThread::~SimpleThread()                                           { }



/********************************************************************************/
/*                                                                              */
/*      SimpleThread creation methods                                           */
/*                                                                              */
/********************************************************************************/


static void *
default_create_rtn(void * ud)
{
   SimpleThread * th = (SimpleThread *) ud;

   int sts = th->run();

   return (void *) sts;
}



bool
SimpleThread::createThread(bool suspended,int stksz)
{
   int sts;
   int fgs = (suspended ? THR_SUSPENDED : 0);

   thread_id = 0;

   sts = thr_create(NULL,stksz,default_create_rtn,this,fgs,&thread_id);

   return (sts == 0);
}



/********************************************************************************/
/*                                                                              */
/*      SimpleThread priority routines                                          */
/*                                                                              */
/********************************************************************************/


int
SimpleThread::getThreadPriority()
{
   int p = 0;

   thr_getprio(thread_id,&p);

   return p;
}


bool
SimpleThread::setThreadPriority(int p)
{
   return thr_setprio(thread_id,p) == 0;
}



/********************************************************************************/
/*                                                                              */
/*      SimpleThread suspend/resume methods                                     */
/*                                                                              */
/********************************************************************************/


int
SimpleThread::suspendThread()
{
   return thr_suspend(thread_id) == 0;
}



int
SimpleThread::resumeThread()
{
   return thr_continue(thread_id) == 0;
}



/********************************************************************************/
/*                                                                              */
/*      SimpleThread dummy run methods                                          */
/*                                                                              */
/********************************************************************************/


int
SimpleThread::run()                                     { return 0; }




/********************************************************************************/
/*                                                                              */
/*      SimpleMutex constructors/destructors                                    */
/*                                                                              */
/********************************************************************************/


SimpleMutex::SimpleMutex(int on)
{
   mutex_init(&mutex_id,USYNC_THREAD,NULL);

   if (on) lock();
}



SimpleMutex::~SimpleMutex()
{
   mutex_destroy(&mutex_id);
}



bool
SimpleMutex::lock()
{
   return mutex_lock(&mutex_id) == 0;
}



bool
SimpleMutex::unlock()
{
   return mutex_unlock(&mutex_id) == 0;
}



/********************************************************************************/
/*                                                                              */
/*      SimpleSemaphore methods                                                 */
/*                                                                              */
/********************************************************************************/


SimpleSemaphore::SimpleSemaphore(unsigned int count)
{
   sema_init(&sema_id,count,USYNC_THREAD,NULL);
}


SimpleSemaphore::~SimpleSemaphore()
{
   sema_destroy(&sema_id);
}


bool
SimpleSemaphore::lock()
{
   return sema_wait(&sema_id) == 0;
}


bool
SimpleSemaphore::unlock()
{
   return sema_post(&sema_id) == 0;
}




/* end of simplethread.cxx */
