/********************************************************************************/
/*                                                                              */
/*              socket_local                                                    */
/*                                                                              */
/*      Local definitions for standard socket implementation                    */
/*                                                                              */
/********************************************************************************/


#ifndef SOCKET_LOCAL_ALREADY_INCLUDED
#define SOCKET_LOCAL_ALREADY_INCLUDED


#include "simple_socket.H"

#include "stddef.H"

#include <signal.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <pwd.h>
#include <grp.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/errno.h>
#include <sys/file.h>
#include <netdb.h>
#include <sgtty.h>
#include <unistd.h>



/********************************************************************************/
/*                                                                              */
/*      Forward Type Definitions                                                */
/*                                                                              */
/********************************************************************************/


typedef class SimpleSocketX11Info *     SimpleSocketX11;



/********************************************************************************/
/*                                                                              */
/*      SimpleSocketX11Callback -- info on a X11 callback                       */
/*                                                                              */
/********************************************************************************/


struct SimpleSocketX11Callback {
   XtInputId x11_id;
   SimpleSocket for_socket;
   int for_fd;

   SimpleSocketX11Callback(XtInputId id,SimpleSocket s,int fd)
      { x11_id = id; for_socket = s; for_fd = fd; }
};


typedef list<SimpleSocketX11Callback>         SimpleSocketX11CallbackList;
typedef SimpleSocketX11CallbackList::iterator SimpleSocketX11CallbackListIter;


class SimpleSocketRemovePred {

private:
   SimpleSocket for_socket;
   XtInputId for_id;

public:
   SimpleSocketRemovePred(SimpleSocket s,XtInputId id);

   int operator () (SimpleSocketX11Callback& cb);
};



/********************************************************************************/
/*                                                                              */
/*      SimpleSocketX11Info -- X11 interface data                               */
/*                                                                              */
/********************************************************************************/


class SimpleSocketX11Info {

protected:
   XtAppContext x11_context;
   SimpleSocketX11CallbackList callback_list;

public:
   SimpleSocketX11Info(XtAppContext = NULL);
   ~SimpleSocketX11Info();

   void addAccept(int fd,SimpleSocket);
   void addRead(int fd,SimpleSocket);
   void addWrite(int fd,SimpleSocket);
   void remove(SimpleSocket,XtInputId = 0);

   void waitForEvent(int pending = 0);

   XtAppContext context() const                         { return x11_context; }

private:
   static void handleAccept(void *,int*,XtInputId *);
   static void handleRead(void *,int*,XtInputId *);
   static void handleWrite(void *,int*,XtInputId *);
};




#endif

/* end of socket_local.H */
