/********************************************************************************/
/*                                                                              */
/*              tchatserver.cxx                                                 */
/*                                                                              */
/*      Server implementation for chat program                                  */
/*                                                                              */
/********************************************************************************/


#include "tchat_server.H"



/********************************************************************************/
/*                                                                              */
/*      main -- main server program                                             */
/*                                                                              */
/********************************************************************************/


main(int argc,const char ** argv)
{
   TChatServer cs = new TChatServerInfo(argc,argv);

   cs->process();

   delete cs;

   return 0;
}




/********************************************************************************/
/*                                                                              */
/*      TChatServerInfo constructors/destructores                               */
/*                                                                              */
/********************************************************************************/


TChatServerInfo::TChatServerInfo(int argc,const char ** argv)
{
   x11_context.defaultContext();
   ConnectInfo::setX11Context(x11_context);
   server_connect = NULL;
}



TChatServerInfo::~TChatServerInfo()
{
   delete server_connect;
}



/********************************************************************************/
/*                                                                              */
/*      TChatServerInfo processing methods                                      */
/*                                                                              */
/********************************************************************************/


void
TChatServerInfo::process()
{
   server_connect = new TChatConnectInfo(this);
   server_connect->openServer(CHAT_SERVER_FILE);

   x11_context.mainLoop();
}




/********************************************************************************/
/*                                                                              */
/*      TChatServerInfo user management methods                                 */
/*                                                                              */
/********************************************************************************/


void
TChatServerInfo::addUser(TChatUser cu)
{
   TChatUserListIter uli;
   TChatCommand cmd;

   server_mutex.lock();

   for (uli = active_list.begin(); uli != active_list.end(); ++uli) {
      cmd = new TChatCommandInfo(TChatCommandInfo::USER_START);
      (*uli)->queueCommand(cmd);
    }

   active_list.push_back(cu);

   server_mutex.unlock();
}


void
TChatServerInfo::removeUser(TChatUser cu)
{
   TChatUserListIter uli;
   TChatCommand cmd;

   server_mutex.lock();

   active_list.remove(cu);

   for (uli = active_list.begin(); uli != active_list.end(); ++uli) {
      cmd = new TChatCommandInfo(TChatCommandInfo::USER_END);
      (*uli)->queueCommand(cmd);
    }

   if (active_list.empty()) exit(0);

   server_mutex.unlock();
}


void
TChatServerInfo::getMessage(TChatUser cu,ConstText msg)
{
   TChatUserListIter uli;
   TChatCommand cmd;

   server_mutex.lock();

   for (uli = active_list.begin(); uli != active_list.end(); ++uli) {
      cmd = new TChatCommandInfo(TChatCommandInfo::MSG_TO,msg);
      (*uli)->queueCommand(cmd);
    }

   server_mutex.unlock();
}



/********************************************************************************/
/*                                                                              */
/*      TChatServerInfo access methods                                          */
/*                                                                              */
/********************************************************************************/


Connect
TChatServerInfo::masterConnect() const
{
   return server_connect;
}



/********************************************************************************/
/*                                                                              */
/*      TChatConnectInfo constructors/destructors                               */
/*                                                                              */
/********************************************************************************/


TChatConnectInfo::TChatConnectInfo(TChatServer cs)
{
   for_server = cs;
}



TChatConnectInfo::~TChatConnectInfo()
{ }



/********************************************************************************/
/*                                                                              */
/*      TChatConnectInfo callback methods                                       */
/*                                                                              */
/********************************************************************************/


void
TChatConnectInfo::acceptCallback()
{
   TChatUser cu = new TChatUserInfo(for_server);
}



/* end of chatserver.cxx */
